<?

function protection_check_htpasswd(){
    if (!in_array(php_sapi_name(), array('apache2', 'apache2handler', 'apache', 'cgi')))
        return 'This method works under Apache webserver only';
}
function protection_check_mod_rewrite(){
    global $config;
    if (!function_exists('virtual'))
        return "PHP must be compiled-in as Apache module to use this method";
    $ret = get_url($url = "$config[root_url]/data/tests/mod_rewrite/1.php");
    if (!strlen($ret))
        return "Unable to fetch result of test from $url";
    elseif ($ret != '2-second_file')
        return "RewriteRule doesn't work in .htaccess files on this server";
    if (is_lite())
        return "This method of protection is not available in aMember Lite";
}

function protection_check_auto_prepend_file(){
    global $config;
    $ret = get_url($url = "$config[root_url]/data/tests/auto_prepend_file/1.htm");
    if (!strlen($ret))
        return "Unable to fetch result of test from $url";
    elseif ($ret != '2-second_file1-first_file')
        return "php_value auto_prepend_file directive doesn't work in .htaccess files on this server";
    if (is_lite())
        return "This method of protection is not available in aMember Lite";
}


function get_protection_methods(){
    $methods = array(
    'htpasswd' => array(
        'name' => 'htpasswd',
        'title' => 'htpasswd',
        'disabled' => protection_check_htpasswd(),
        'description' => 'Usual htpasswd protection. Popup login box will be
        displayed when member enters into protected area, and it is impossible to
        avoid this box with this method of protection. Any type of content can be protected.
        Special configuration required to track customers access and use account sharing protection
        '
    ),
    'mod_rewrite' => array(
        'name' => 'mod_rewrite',
        'title' => 'php_include+mod_rewrite',
        'disabled' => protection_check_mod_rewrite(),
        'description' => 
        'Most advanced method of protection - user will see your custom HTML login form.
        This method can protect any type of content, but may cause problems with
        complex PHP scripts residing in protected area. Remember, you do not need to
        specially protect third-party scripts if you have enabled aMember integration
        plugin for it. For example, if you are using vBulletin-integration plugin, you do not
        need to protect vBulletin using this tool - just use vBulletin permission management 
        controls.<br>
        Access sharing prevention works automatically with this plugin.'
    ),
    'auto_prepend_file' => array(
        'name' => 'auto_prepend_file',
        'title' => 'php_include+auto_prepend_file',
        'disabled' => protection_check_auto_prepend_file(),
        'description' => 
        'This method can protect only PHP and HTML files, not images. It may cause problems with
        SSI files (files having .shtml extenstion). This method does not protect
        images or downloadable content in the protected folder - only PHP and HTML files.<br>
        Access sharing prevention works automatically with this plugin.'
    ),
    'php_include' => array(
        'name' => 'php_include',
        'title' => 'php_include',
        'disabled' => 0,
        'description' => 
        'This method can protect only PHP files and applications, nothing else.
        This method does not protect  images, HTML files or downloadable content 
        in the protected folder - it protects PHP files only.<br>
        Access sharing prevention works automatically with this plugin.<br>
        <i>You have to manually install this protection (edit your PHP files)</i>'
    )
    );
    return $methods;
}


function validate_htaccess($vars){
    if (!is_dir($vars['path'])){
        return "Path '$vars[path]' is not a folder";
    }
    $f = "$vars[path]/.htaccess";
    if (!file_exists($f) && !is_writable($vars['path'])){
        return "Folder $vars[path] is not a writable for the PHP script. Please <br>
        chmod this folder using webhosting control panel file manager or using your<br>
        favorite FTP client to 777 (write, read and execute for all)<br>
        After protection, please don't forget to chmod it back to 755.
        ";
    }
    if (file_exists($f) && 
        !preg_match('/^\s*#+ AMEMBER START #+.+AMEMBER FINISH #+\s*$/s',
            join('',file($f)))
    ){
        return "File $f is exists and contains not only aMember code, so<br>
        aMember cannot modify it. Delete this file if you don't need this file,
        and if you want aMember to create a new file instead.";
    }

    if (file_exists($f) && !is_writeable($f)){
        return "File $f is not a writable for the PHP script. Please <br>
        chmod this file using webhosting control panel file manager or using your<br>
        favorite FTP client to 666 (write and read for all)<br>
        After protection, please don't forget to chmod it back to 644.
        ";
    }
}

function write_files($vars, $files){
    foreach ($files as $fname => $content){
        $f = "$vars[path]/$fname";
        $fh = fopen($f, 'w');
        if (!$fh)
            return "Cannot open file $f for write";
        fwrite($fh, $content);        
        if (!fclose($fh)) 
            return "Cannot write file $f";
    }
}

function protect_htpasswd($vars, &$files){
    global $config;

    if ($err = validate_htaccess($vars))
        return $err;

    if ($vars['product_id_all']){
        $require = "valid-user";
    } else {
        $require = ' group ';
        foreach ($vars['product_id'] as $i) $require .= " PRODUCT_$i";
    }
    $files['.htaccess'] = <<<CUT
########### AMEMBER START #####################
AuthType Basic
AuthName "Members Only"
AuthUserFile {$config[root_dir]}/data/.htpasswd
AuthGroupFile {$config[root_dir]}/data/.htgroup
Require $require
########### AMEMBER FINISH ####################
CUT;

    if ($err = write_files($vars, $files))
        return $err;
}



function protect_mod_rewrite($vars, &$files){
    global $config;
    if ($err = validate_htaccess($vars))
        return $err;
    $ru = parse_url($config['root_url']);
    if ($vars['product_id_all'])
        $line = "";
    else
        $line = '[E=PRODUCT_ID:' . join(';', $vars['product_id']) . ']';
    $files['.htaccess'] = <<<CUT
########### AMEMBER START #####################
Options +FollowSymLinks
RewriteEngine On
RewriteRule ^(.*)$ {$ru[path]}/plugins/protect/php_include/rewrite.php $line
########### AMEMBER FINISH ####################
CUT;

    if ($err = write_files($vars, $files))
        return $err;
}

function protect_auto_prepend_file($vars, &$files){
    global $config;
    if ($err = validate_htaccess($vars))
        return $err;
    if ($vars['product_id_all'])
        $line = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24";
    else
        $line = join(',', $vars['product_id']);
    $files['.htaccess'] = <<<CUT
########### AMEMBER START #####################
AddType application/x-httpd-php .html
AddType application/x-httpd-php .htm 
php_value auto_prepend_file $vars[path]/amember_protect.inc.php
########### AMEMBER FINISH ####################
CUT;
    

    $files['amember_protect.inc.php'] = <<<CUT
<?php
\$_product_id = array($line); 
require_once '{$config[root_dir]}/plugins/protect/php_include/check.inc.php';
?>
CUT;
    if ($err = write_files($vars, $files))
        return $err;
}
function protect_php_include($vars, &$files){
    global $config;
    if ($vars['product_id_all'])
        $line = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24";
    else
        $line = join(',', $vars['product_id']);
    $s = <<<CUT
CUT;
    print <<<CUT
    <html><head>
    This method of protection is simple - your must edit your PHP files and
    include to TOP of each PHP file (or just into one which is included by 
    all other files) these lines of code:    
    <table bgcolor=gray><tr><td bgcolor=gray><pre>&lt;?php
\$_product_id = array($line); 
require_once '{$config[root_dir]}/plugins/protect/php_include/check.inc.php';
?&gt;</pre></td></tr></table>

    <br><br>Click <a href='protect.php'>here</a> to continue...
CUT;
    $GLOBALS['protection_is_instruction']=1;
}

?>