<?php                          
/**
* This file contain commonly used subroutines
*
*     Author: Alex Scott
*      Email: alex@cgi-central.net
*        Web: http://www.cgi-central.net
*    Details: Common Routines
*    FileName $RCSfile: common.inc.php,v $
*    Release: 2.1.8PRO ($Revision: 1.17.2.6.4.38 $)
*
* Please direct bug reports,suggestions or feedback to the cgi-central forums.
* http://www.cgi-central.net/forum/
*                                                                          
*
* aMember PRO is a commercial software. Any distribution is strictly prohibited.
*                                                                                 
*/

set_magic_quotes_runtime(0);

$config['version'] = '2.1.8PRO';

$config['tables']  = array(
    'access_log',
    'cron_run',
    'config',
    'coupon',
    'error_log',
    'members',
    'payments',
    'products'
);

$config['tables_skip_backup']  = array(
    'access_log',
    'error_log'
);


/**
* Create new Smarty object 
* @return double Smarty to newly created Smarty object
*
*/
function smarty_prefilter_literal_script($source, &$smarty){
    $result=&$source;
    $result=preg_replace('~<script\b(?![^>]*smarty)~iU', '<!--{literal} --><script', $result);
    $result=preg_replace('~</script>~iU', '</script><!--{literal} {/literal}-->', $result);
    return $result;
}
function smarty_prefilter_literal_style($source, &$smarty){
    $result=&$source;
    $result=preg_replace('~<style\b(?![^>]*smarty)~iU', '<!--{literal} --><style', $result);
    $result=preg_replace('~</style>~iU', '</style><!--{literal} {/literal}-->', $result);
    return $result;
}
function smarty_prefilter_put_config_cb($args){
    global $config;
    return $config[$args[1]];
}
function smarty_outputfilter_put_config($source, &$smarty){
    $result=&$source;
    $result=preg_replace_callback('/#\$config\.(.+?)#/', 'smarty_prefilter_put_config_cb',
        $result);
    return $result;
}
function smarty_outputfilter_literal_cleanup($source, &$smarty){
    $result=&$source;
    $result=preg_replace('~<!--({literal})? -->~iU', '', $result);
    return $result;
}

function new_smarty(){
    global $config;
    switch ($config['tpl_cache_level']){
    case '2':
        $t = new _Smarty();
        $t->force_compile = 0;
        break;
    case '1': 
        require_once($config['root_dir'].'/smarty/SmartySingleDir.class.php');
        $t = new _SmartySingleDir();
        $t->force_compile = 0;
        break;
    case '0': default:
        require_once($config['root_dir'].'/smarty/SmartyNoWrite.class.php');
        $t = new _SmartyNoWrite();
        $t->force_compile = 1;
        $t->compile_check = 1;
        break;
    }
    $t->register_prefilter('smarty_prefilter_literal_script');
    $t->register_prefilter('smarty_prefilter_literal_style');
    $t->register_outputfilter('smarty_outputfilter_put_config');
    $t->register_outputfilter('smarty_outputfilter_literal_cleanup');
    $t->template_dir = "$config[root_dir]/templates";
    $t->compile_dir  = "$config[root_dir]/templates_c";

    ///
    $t->assign('config', $config);
    return $t;
}

/**
* Check email using regexes
* @param string email
* @return bool true if email valid, false if not
*/

function check_email($email) {
    #characters allowed on name: 0-9a-Z-._ on host: 0-9a-Z-. on between: @
    if (!preg_match('/^[0-9a-zA-Z\.\-\_]+\@[0-9a-zA-Z\.\-]+$/', $email))
        return false;

    #must start or end with alpha or num
    if ( preg_match('/^[^0-9a-zA-Z]|[^0-9a-zA-Z]$/', $email))
        return false;

    #name must end with alpha or num
    if (!preg_match('/([0-9a-zA-Z_]{1})\@./',$email) )                    
        return false;

    #host must start with alpha or num
    if (!preg_match('/.\@([0-9a-zA-Z_]{1})/',$email) )                    
        return false;

    #pair .- or -. or -- or .. not allowed
    if ( preg_match('/.\.\-.|.\-\..|.\.\..|.\-\-./',$email) )
        return false;

    #pair ._ or -_ or _. or _- or __ not allowed
    if ( preg_match('/.\.\_.|.\-\_.|.\_\..|.\_\-.|.\_\_./',$email) )
        return false;

    #host must end with '.' plus 2-5 alpha for TopLevelDomain
    if (!preg_match('/\.([a-zA-Z]{2,5})$/',$email) )
        return false;

    return true;
}

/**
* Retrieve input vars, trim spaces and return as array
* @return array array of input vars (HTTP_POST_VARS or HTTP_GET_VARS)
*
*/
function get_input_vars(){
    global $HTTP_SERVER_VARS;
    $REQUEST_METHOD = $HTTP_SERVER_VARS['REQUEST_METHOD'];
    global $HTTP_POST_VARS;
    global $HTTP_GET_VARS;

    $vars = $REQUEST_METHOD == 'POST' ? $HTTP_POST_VARS : $HTTP_GET_VARS;
    foreach ($vars as $k=>$v){
        if (is_array($v)) continue;
        if (get_magic_quotes_gpc()) $v = stripslashes($v);
        $vars[$k] = trim($v);
    }
    return $vars;
}

/**
* Display fatal error to user. 
* Should send mail and save log (not implemented yet)
* Exit current script
* return bool should never return
*/
function fatal_error($error, $log_error=1){
    global $config, $db;
    global $in_fatal_error; //!
    $in_fatal_error++;
    if ($in_fatal_error > 2){
        die("<br><br><b>fatal_error called twice</b>");
    }
    $t = & new_smarty();    
    $t->assign('error', $error);
    $t->assign('admin_email', $config['admin_email']);
    $t->display("fatal_error.html");
    // log error
    if ($log_error)
        $db->log_error($error);
    exit();
}



/**
* Load URL
* if post specified, will be POST, if not, will be GET request
* return string with result of fetch
*/
function get_url($url, $post=''){
    global $db;
    if (extension_loaded("curl")){
        $ch=curl_init($url);
        curl_setopt($ch, CURLOPT_HEADER, 0);
        if ($post)  {
            curl_setopt($ch, CURLOPT_POSTFIELDS, $post);  
        }
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);    
        $buffer = curl_exec($ch);                       
        curl_close($ch);
        return $buffer;
    } else {
        global $config;
        $curl = $config['curl'];
        if (!strlen($curl)) {
            $db->log_error("cURL path is not set - cc transaction cannot be completed");
            return;
        }
        if (substr(php_uname(), 0, 7) == "Windows") {
            if ($post)
            $ret = `$curl -d "$post" "$url"`;
            else
            $ret = `$curl "$url"`;
        } else {
            $url  = escapeshellarg($url);
            $post = escapeshellarg($post);
            if ($post)
            $ret = `$curl -d $post $url`;
            else
            $ret = `$curl $url`;
        }
        return $ret;
    }
}



function set_date_from_smarty($prefix, &$vars){
    return $vars[$prefix] = sprintf('%04d-%02d-%02d', 
        $vars[$prefix.  'Year'],
        $vars[$prefix.  'Month'],
        $vars[$prefix.  'Day']
    );
}

function mail_signup_user($member_id){
    global $t, $db, $config;
    if (!is_object($t))
        $t = & new_smarty();
    $u = $db->get_user($member_id);
    $u['name'] = $u['name_f'] . ' ' . $u['name_l'];
    ///////////////////////////////////
    $t->assign('login', $u['login']);
    $t->assign('pass',  $u['pass']);
    $t->assign('name_f', $u['name_f']);
    $t->assign('name_l', $u['name_l']);
    ///////////////////////////////////
    $t->assign('user', $u);
    ///////////////////////////////////
    $mail = $t->fetch("signup_mail.txt");
    mail_customer($u['email'], $mail, "Signup Completed");
}

function mail_pending_user($member_id, $payment_id){
    global $t, $db, $config;
    if (!is_object($t))
        $t = & new_smarty();
    $p = $db->get_payment($payment_id);
    $u = $db->get_user($member_id);
    ///////////////////////////////////
    $t->assign('login', $u['login']);
    $t->assign('pass',  $u['pass']);
    $t->assign('name_f', $u['name_f']);
    $t->assign('name_l', $u['name_l']);
    ///////////////////////////////////
    $t->assign('user', $u);
    $t->assign('payment', $p);
    ///////////////////////////////////
    $mail = $t->fetch("pending_mail.txt");
    mail_customer($u['email'], $mail, "Pending Payment");
}

function mail_pending_admin($member_id, $payment_id){
    global $t, $db, $config;
    if (!is_object($t))
        $t = & new_smarty();
    $p = $db->get_payment($payment_id);
    $u = $db->get_user($member_id);
    ///////////////////////////////////
    $t->assign('login', $u['login']);
    $t->assign('pass',  $u['pass']);
    $t->assign('name_f', $u['name_f']);
    $t->assign('name_l', $u['name_l']);
    ///////////////////////////////////
    $t->assign('user', $u);
    $t->assign('payment', $p);
    ///////////////////////////////////
    $mail = $t->fetch("admin/pending_mail.txt");
    mail_customer($config['admin_email'], $mail, "** Pending Payment");
}

function mail_approval_wait_user($member_id){
    global $t, $db, $config;
    if (!is_object($t))
        $t = & new_smarty();
    $u = $db->get_user($member_id);
    ///////////////////////////////////
    $t->assign('login', $u['login']);
    $t->assign('pass',  $u['pass']);
    $t->assign('name_f', $u['name_f']);
    $t->assign('name_l', $u['name_l']);
    ///////////////////////////////////
    $t->assign('user', $u);
    ///////////////////////////////////
    $mail = $t->fetch("approval_mail.txt");
    mail_customer($u['email'], $mail, "Your signup is pending");
}

function mail_approval_wait_admin($member_id){
    global $t, $db, $config;
    $u = $db->get_user($member_id);
    mail_admin("
    New signup completed and need your approval. 
    Please review it here:
        $config[root_url]/admin/users.php?member_id=$member_id&action=edit

    Short details:
        username: $u[login]
        email:    $u[email]
        name:     $u[name_f] $u[name_l]
    --
    Your aMember Pro script 
    $config[root_url]/admin
    ", "*** Your approval required");
}

function mail_payment_admin($payment_id, $member_id){
    global $t, $db, $config;
    $p = $db->get_payment($payment_id);
    $u = $db->get_user($p['member_id']);
    $pr = $db->get_product($p['product_id']);
    if (!$t) $t = &new_smarty();
    $t->assign('user',    $u);
    $t->assign('payment', $p);
    $t->assign('product', $pr);
    $mail = $t->fetch('admin/mail_payment.txt');
    mail_admin($mail, "*** New Payment");
}

function mail_rebill_failed_member($user, $payment_id, $product, $error){
    global $t, $db;
    if (!$t) $t = &new_smarty();
    $t->assign('user',    $user);
    $t->assign('payment', $db->get_payment($payment_id));
    $t->assign('product', $product->config);
    $t->assign('error',   $error);
    $mail = $t->fetch('cc/rebill_failed.txt');
    mail_customer($user['email'], $mail, 'Your subscription renewal failed');
}

function mail_rebill_success_member($user, $payment_id, $product){
    global $t, $db;
    if (!$t) $t = &new_smarty();
    $t->assign('user',    $user);
    $t->assign('payment', $db->get_payment($payment_id));
    $t->assign('product', $product->config);
    $mail = $t->fetch('cc/rebill_success.txt');
    mail_customer($user['email'], $mail, 'Your subscription renewed');
}

function mail_verification_email($user, $url){
    global $t, $db;
    if (!$t) $t = &new_smarty();
    $t->assign('user',    $user);
    $t->assign('url',     $url);
    $mail = $t->fetch('email_verify.txt');
    mail_customer($user['email'], $mail, 'Email address verification');
}

function determine_mime_type($filename){
    $common_mime_types = array(
         "exe" => "application/octet-stream",
         "class" => "application/octet-stream",
         "so" => "application/octet-stream",
         "dll" => "application/octet-stream",
         "oda" => "application/oda",
         "hqx" => "application/mac-binhex40",
         "cpt" => "application/mac-compactpro",
         "doc" => "application/msword",
         "bin" => "application/octet-stream",
         "dms" => "application/octet-stream",
         "lha" => "application/octet-stream",
         "lzh" => "application/octet-stream",
         "pdf" => "application/pdf",
         "ai" => "application/postscript",
         "eps" => "application/postscript",
         "ps" => "application/postscript",
         "smi" => "application/smil",
         "smil" => "application/smil",
         "bcpio" => "application/x-bcpio",
         "wbxml" => "application/vnd.wap.wbxml",
         "wmlc" => "application/vnd.wap.wmlc",
         "wmlsc" => "application/vnd.wap.wmlscriptc",
         "vcd" => "application/x-cdlink",
         "pgn" => "application/x-chess-pgn",
         "cpio" => "application/x-cpio",
         "csh" => "application/x-csh",
         "dcr" => "application/x-director",
         "dir" => "application/x-director",
         "dxr" => "application/x-director",
         "dvi" => "application/x-dvi",
         "spl" => "application/x-futuresplash",
         "gtar" => "application/x-gtar",
         "hdf" => "application/x-hdf",
         "skp" => "application/x-koan",
         "skd" => "application/x-koan",
         "js" => "application/x-javascript",
         "skt" => "application/x-koan",
         "skm" => "application/x-koan",
         "latex" => "application/x-latex",
         "nc" => "application/x-netcdf",
         "cdf" => "application/x-netcdf",
         "sh" => "application/x-sh",
         "shar" => "application/x-shar",
         "swf" => "application/x-shockwave-flash",
         "sit" => "application/x-stuffit",
         "sv4cpio" => "application/x-sv4cpio",
         "sv4crc" => "application/x-sv4crc",
         "tar" => "application/x-tar",
         "tcl" => "application/x-tcl",
         "tex" => "application/x-tex",
         "texinfo" => "application/x-texinfo",
         "texi" => "application/x-texinfo",
         "t" => "application/x-troff",
         "tr" => "application/x-troff",
         "roff" => "application/x-troff",
         "man" => "application/x-troff-man",
         "me" => "application/x-troff-me",
         "ms" => "application/x-troff-ms",
         "ustar" => "application/x-ustar",
         "src" => "application/x-wais-source",
         "xhtml" => "application/xhtml+xml",
         "xht" => "application/xhtml+xml",
         "zip" => "application/zip",
         "au" => "audio/basic",
         "snd" => "audio/basic",
         "mid" => "audio/midi",
         "midi" => "audio/midi",
         "kar" => "audio/midi",
         "mpga" => "audio/mpeg",
         "mp2" => "audio/mpeg",
         "mp3" => "audio/mpeg",
         "aif" => "audio/x-aiff",
         "aiff" => "audio/x-aiff",
         "aifc" => "audio/x-aiff",
         "m3u" => "audio/x-mpegurl",
         "ram" => "audio/x-pn-realaudio",
         "rm" => "audio/x-pn-realaudio",
         "rpm" => "audio/x-pn-realaudio-plugin",
         "ra" => "audio/x-realaudio",
         "wav" => "audio/x-wav",
         "pdb" => "chemical/x-pdb",
         "xyz" => "chemical/x-xyz",
         "bmp" => "image/bmp",
         "gif" => "image/gif",
         "ief" => "image/ief",
         "jpeg" => "image/jpeg",
         "jpg" => "image/jpeg",
         "jpe" => "image/jpeg",
         "png" => "image/png",
         "tiff" => "image/tiff",
         "tif" => "image/tif",
         "djvu" => "image/vnd.djvu",
         "djv" => "image/vnd.djvu",
         "wbmp" => "image/vnd.wap.wbmp",
         "ras" => "image/x-cmu-raster",
         "pnm" => "image/x-portable-anymap",
         "pbm" => "image/x-portable-bitmap",
         "pgm" => "image/x-portable-graymap",
         "ppm" => "image/x-portable-pixmap",
         "rgb" => "image/x-rgb",
         "xbm" => "image/x-xbitmap",
         "xpm" => "image/x-xpixmap",
         "xwd" => "image/x-windowdump",
         "igs" => "model/iges",
         "iges" => "model/iges",
         "msh" => "model/mesh",
         "mesh" => "model/mesh",
         "silo" => "model/mesh",
         "wrl" => "model/vrml",
         "vrml" => "model/vrml",
         "mpeg" => "video/mpeg",
         "mpg" => "video/mpeg",
         "mpe" => "video/mpeg",
         "qt" => "video/quicktime",
         "mov" => "video/quicktime",
         "mxu" => "video/vnd.mpegurl",
         "avi" => "video/x-msvideo",
         "movie" => "video/x-sgi-movie",
         "css" => "text/css",
         "asc" => "text/plain",
         "txt" => "text/plain",
         "rtx" => "text/richtext",
         "rtf" => "text/rtf",
         "sgml" => "text/sgml",
         "sgm" => "text/sgml",
         "tsv" => "text/tab-seperated-values",
         "wml" => "text/vnd.wap.wml",
         "wmls" => "text/vnd.wap.wmlscript",
         "etx" => "text/x-setext",
         "xml" => "text/xml",
         "xsl" => "text/xml",

         "htm" => "text/html",
         "html" => "text/html",
         "shtml" => "text/html"
    );

    if (!preg_match('/\.([\w\d]+)$/', $filename, $regs)) 
        return 'application/octet-stream';
    $ext = $regs[1];
    if ($mime = $common_mime_types[$ext])
        return $mime;
    else
        return 'application/octet-stream';
}

function sql_to_timestamp($sqldate){
    list($y,$m,$d) = split('-', $sqldate);
    return mktime(0,0,0,$m,$d,$y);
}

function mail_customer($email, $text, $subject='', $is_html=0, $attachments=''){
    global $config, $db;
    if (!strlen($email)) 
        return;
    if (preg_match('/^Subject: (.+?)[\n\r]+/im', $text, $args)){
        // found subject in body of message! then save it and remove from 
        // message
        $subject = $args[1];
        $text = preg_replace('/(^Subject: .+?[\n\r]+)/im', '', $text);
    }
    if (preg_match('/^Format: (\w+?)\s*$/im', $text, $args)){
        $format = $args[1];
        if (!strcasecmp('HTML', $format)){
            $is_html = 1;
            $text = preg_replace('/^Format: (\w+?)\s*$/im', '', $text);
        } else if (!strcasecmp('TEXT', $format)){
            $is_html = 0;
            $text = preg_replace('/^Format: (\w+?)\s*$/im', '', $text);
        } 
    }
    if (preg_match_all('/^Attachment: (.+?)\s*$/im', $text, $args)){
        foreach ($args[1] as $fname){
            if ($fname[0] != '/')
                $fname = $config['root_dir'] . '/templates/' . $fname;
            if (!file_exists($fname))
                $db->log_error("Email attachment file : '$fname' is not exists - check your e-mail templates");
            elseif (!is_readable($fname))
                $db->log_error("Email attachment file : '$fname' is not readable for the script - check your e-mail templates and/or chmod file");
            else
                $attachments[] = array(
                   'name'     => basename($fname),
                   'type'     => determine_mime_type(basename($fname)),
                   'tmp_name' => $fname
                );
        }
        $text = preg_replace('/^Attachment: (.+?)\s*$/im', '', $text);
    }

    if (preg_match('/\<(.+?)\>\s*$/', $email, $regs))
        $email_only = $regs[1];
    else
        $email_only = $email;

    $headers = array(
        "From: $config[admin_email]",
       "X-Mailer: aMember (http://cgi-central.net/)" 
    );
//    if ($email != $email_only) 
//        $headers[] = "To: $email";
    if ($attachments){
        $mime_delimiter = "----=_NextPart_000_0001_".md5(time());
        $headers[] = "MIME-Version: 1.0";
        $headers[] = "Content-Type: multipart/mixed;";
        $headers[] = "    boundary=\"$mime_delimiter\"";
        
        $text = 
"This is a multi-part message in MIME format.
    
--$mime_delimiter
Content-type: ".($is_html ? "text/html" : "text/plain")."
Content-Transfer-Encoding: 8bit
Content-Disposition: inline

$text";
        foreach ((array)$attachments as $a){
            $file_name     = $a['name'];
            $file_type     = $a['type'];
            $file_tmp_name = $a['tmp_name'];
            $file_cnt = "";
            $f=@fopen($file_tmp_name, "rb");
            if (!$f) continue;
            while($f && !feof($f))
                $file_cnt .= fread($f, 4096);
            fclose($f);
            if (!strlen($file_type)) 
                $file_type="applicaton/octet-stream";
            if ($file_type == 'application/x-msdownload')
                $file_type = "applicaton/octet-stream";

            $text .= "\n--$mime_delimiter\n";
            $text .= "Content-Type: $file_type;\n       name=\"$file_name\"\n";
            $text .= "Content-Transfer-Encoding: base64\n";
            $text .= "Content-Disposition: attachment;\n       filename=\"$file_name\"\n\n";
            $text .= chunk_split(base64_encode($file_cnt));
        }
        $text .= "\n--$mime_delimiter--";
    } elseif ($is_html){
        $headers[] = "MIME-Version: 1.0";
        $headers[] = "Content-type: text/html";
    }
    $headers = join("\n", $headers);


   mail($email_only, $subject, $text, $headers);
}

function mail_admin($text, $subject=''){
    global $config;
    $email = $config['admin_email'];
    if (preg_match('/^Subject: (.+?)[\n\r]+/im', $text, $args)){
        // found subject in body of message! then save it and remove from 
        // message
        $subject = $args[1];
        $text = preg_replace('/(^Subject: .+?[\n\r]+)/im', '', $text);
    }
    mail($email, $subject, $text, 
        "From: $config[admin_email]\r\n"
       ."X-Mailer: aMember (http://cgi-central.net/)" 
    );
}

// output html code and possible header, suitable for redirect
function html_redirect($url, $print_header=0, $title='', $text=''){
    global $t;
    if (!$t) $t = &new_smarty();
    $t->assign('title', $title);
    $t->assign('text', $text);
    $t->assign('url', $url);
    $t->display('redirect.html');
}

function check_demo($msg="Sorry, this function disabled in the demo"){
    if ($GLOBALS['config']['demo'])
        die($msg);    
}

function compare_with_pattern($pattern, $value){    
    $value = trim($value);
    $pattern = trim($pattern);
    if (!strlen($pattern)) 
        return 0;
    $pattern = preg_quote($pattern);
    $pattern = str_replace('\*', '.+?', $pattern);
    return preg_match("/^$pattern\$/", $value);
}

function add_password_to_url($url, $username_password=''){
    global $db, $config;
    if (!strlen($url)) 
        return "";
    elseif (preg_match('~^(http://|https://)(.+?)(/.*|)$~', $url, $regs)){
        if (preg_match('/\@(.+?)$/', $regs[2], $regs_x)) 
            $regs[2] = $regs_x[1];
        return $regs[1] . ($username_password?"$username_password@":'') . $regs[2] . $regs[3];
    } elseif ($url[0] == '/') {
        $u = parse_url($config['root_url']);
        $s = "$u[scheme]://".($username_password?"$username_password@":'')."$u[host]$url";
        return $s;
    } else { /// url is relative to aMember folder
        $u = parse_url($config['root_url']);
        if (!preg_match('/\/$/', $u['path']))
            $u['path'] .= "/";
        $s = "$u[scheme]://".($username_password?"$username_password@":'')."$u[host]$u[path]$url";
        return $s;
    }
}

function generate_login($vars='') {
    global $db, $config;
    $vars = (array)$vars;

    // try to use first part of email
    if (preg_match("/^([a-zA-Z0-9_]+)\@/", $vars['email'], $regs)){
        $login = $regs[1];
        if (strlen($login) > $config['login_max_length'])
            $login = substr($login, 0, $config['login_max_length']);
        if ($db->check_uniq_login($login) && (strlen($login)>=$config['login_min_length'])){
            return $login;
        }
    }    
    

    // from first and last name
    $fn = strtolower(preg_replace('/[^\w\d_]/', '', $vars['name_f']));
    $ln = strtolower(preg_replace('/[^\w\d_]/', '', $vars['name_l']));
    if ($fn && $ln)
        $login = $fn.'_'.$ln;
    else
        $login = $fn . $ln;
    if (strlen($login) > $config['login_max_length'])
        $login = substr($login, 0, $config['login_max_length']);
    if ($db->check_uniq_login($login) && (strlen($login)>=$config['login_min_length'])){
        return $login;
    }    

    // try to add numbers while free login not found
    if (strlen($login) > $config['login_max_length'])
        $login = substr($login, 0, $config['login_max_length']);
    for ($i=1;$i<999;$i++){
        if (strlen($login) > $config['login_max_length'])
            $login = substr($login, 0, $config['login_max_length']);
        $nlogin = $login . $i;
        if ($db->check_uniq_login($nlogin) && (strlen($login)>$config['login_min_length'])){
            return $nlogin;
        }
    }    

    // will generate it
    // a bit of configuration
    global $config;
    $min_length=$config['login_min_length'] < 4 ? 4 : $config['login_min_length'];
    $max_length=$config['login_max_length'] > 10 ? 10 : $config['login_max_length'];
    $all_g = "aeiyo";
    $all_s = "bcdfghjkmnpqrstwxz";
    /// let's go
    do {
        $pass = "";
        srand((double)microtime()*1000000);
        $length = rand($min_length, $max_length);
        for($i=0;$i<$length;$i++) {
            srand((double)microtime()*1000000);
            if ($i % 2)
                $pass .= $all_g[ rand(0, strlen($all_g) - 1) ];
            else 
                $pass .= $all_s[ rand(0, strlen($all_s) - 1) ];
        }
    } while (!$db->check_uniq_login($pass));
    return $pass;
}

function generate_password($vars=''){
    // a bit of configuration
    global $config;
    $vars = (array)$vars;
    $min_length=$config['pass_min_length'] < 4 ? 4 : $config['pass_min_length'];
    $max_length=$config['pass_max_length'] > 10 ? 10 : $config['pass_max_length'];
    $all_g = "aeiyo";
    $all_gn = $all_g . "1234567890";
    $all_s = "bcdfghjkmnpqrstwxz";
    /// let's go
    $pass = "";
    srand((double)microtime()*1000000);
    $length = rand($min_length, $max_length);
    for($i=0;$i<$length;$i++) {
        srand((double)microtime()*1000000);
        if ($i % 2)
            if ($i < $min_length)
                $pass .= $all_g[ rand(0, strlen($all_g) - 1) ];
            else
                $pass .= $all_gn[ rand(0, strlen($all_gn) - 1) ];
        else 
            $pass .= $all_s[ rand(0, strlen($all_s) - 1) ];
    }
    return $pass;
}

srand((double) microtime() * 1000000); 

$config['state_options'] = array(
    ""    => '[Select State]',
    "XX" => "Outside US and Canada",
    "AL" => "Alabama",
    "AK" => "Alaska",
    "AB" => "Alberta", 
    "AS" => "American Samoa",
    "AZ" => "Arizona",
    "AR" => "Arkansas",
    "AA" => "Armed Forces Americas",
    "AE" => "Armed Forces Europe",
    "AP" => "Armed Forces Pacific",
    "BC" => "British Columbia",   
    "CA" => "California",
    "CO" => "Colorado",
    "CT" => "Connecticut",
    "DE" => "Delaware",
    "DC" => "District Of Columbia",
    "FL" => "Florida",
    "GA" => "Georgia",
    "GU" => "Guam",
    "HI" => "Hawaii",
    "ID" => "Idaho",
    "IL" => "Illinois",
    "IN" => "Indiana",
    "IA" => "Iowa",
    "KS" => "Kansas",
    "KY" => "Kentucky",
    "LA" => "Louisiana",
    "ME" => "Maine",
    "MB" => "Manitoba",   
    "MD" => "Maryland",
    "MA" => "Massachusetts",
    "MI" => "Michigan",
    "MN" => "Minnesota",
    "MS" => "Mississippi",
    "MO" => "Missouri",
    "MT" => "Montana",
    "NE" => "Nebraska",
    "NV" => "Nevada",
    "NB" => "New Brunswick",  
    "NH" => "New Hampshire",
    "NJ" => "New Jersey",
    "NM" => "New Mexico",
    "NY" => "New York",
    "NF" => "Newfoundland",   
    "NC" => "North Carolina",
    "ND" => "North Dakota",
    "MP" => "Northern Mariana Is",
    "NT" => "Northwest Territories",
    "NS" => "Nova Scotia",
    "NU" => "Nunavut",    
    "OH" => "Ohio",
    "OK" => "Oklahoma",
    "ON" => "Ontario",
    "OR" => "Oregon",
    "PW" => "Palau",
    "PA" => "Pennsylvania",
    "PE" => "Prince Edward Island",
    "QC" => "Province du Quebec", 
    "PR" => "Puerto Rico",
    "RI" => "Rhode Island",
    "SK" => "Saskatchewan",   
    "SC" => "South Carolina",
    "SD" => "South Dakota",
    "TN" => "Tennessee",
    "TX" => "Texas",
    "UT" => "Utah",
    "VT" => "Vermont",
    "VI" => "Virgin Islands",
    "VA" => "Virginia",
    "WA" => "Washington",
    "WV" => "West Virginia",
    "WI" => "Wisconsin",
    "WY" => "Wyoming",
    "YT" => "Yukon Territory"
);

$config['country_options'] = array(
    ""   => "[Select Country]",
    "US" => "United States",
    "CA" => "Canada",
    "AF" => "Afghanistan",
    "AL" => "Albania",
    "DZ" => "Algeria",
    "AS" => "American Samoa",
    "AD" => "Andorra",
    "AO" => "Angola",
    "AI" => "Anguilla",
    "AQ" => "Antarctica",
    "AG" => "Antigua and Barbuda",
    "AR" => "Argentina",
    "AM" => "Armenia",
    "AW" => "Aruba",
    "AU" => "Australia",
    "AT" => "Austria",
    "AZ" => "Azerbaijan",
    "BS" => "Bahamas",
    "BH" => "Bahrain",
    "BD" => "Bangladesh",
    "BB" => "Barbados",
    "BY" => "Belarus",
    "BE" => "Belgium",
    "BZ" => "Belize",
    "BJ" => "Benin",
    "BM" => "Bermuda",
    "BT" => "Bhutan",
    "BO" => "Bolivia",
    "BA" => "Bosnia and Herzegovina",
    "BW" => "Botswana",
    "BV" => "Bouvet Island",
    "BR" => "Brazil",
    "IO" => "British Indian Ocean Territory",
    "BN" => "Brunei",
    "BG" => "Bulgaria",
    "BF" => "Burkina Faso",
    "BI" => "Burundi",
    "KH" => "Cambodia",
    "CM" => "Cameroon",
    "CA" => "Canada",
    "CV" => "Cape Verde",
    "KY" => "Cayman Islands",
    "CF" => "Central African Republic",
    "TD" => "Chad",
    "CL" => "Chile",
    "CN" => "China",
    "CX" => "Christmas Island",
    "CC" => "Cocos (Keeling) Islands",
    "CO" => "Colombia",
    "KM" => "Comoros",
    "CG" => "Congo",
    "CK" => "Cook Islands",
    "CR" => "Costa Rica",
    "CI" => "Cote d'Ivoire",
    "HR" => "Croatia (Hrvatska)",
    "CU" => "Cuba",
    "CY" => "Cyprus",
    "CZ" => "Czech Republic",
    "CD" => "Congo (DRC)",
    "DK" => "Denmark",
    "DJ" => "Djibouti",
    "DM" => "Dominica",
    "DO" => "Dominican Republic",
    "TP" => "East Timor",
    "EC" => "Ecuador",
    "EG" => "Egypt",
    "SV" => "El Salvador",
    "GQ" => "Equatorial Guinea",
    "ER" => "Eritrea",
    "EE" => "Estonia",
    "ET" => "Ethiopia",
    "FK" => "Falkland Islands (Islas Malvinas)",
    "FO" => "Faroe Islands",
    "FJ" => "Fiji Islands",
    "FI" => "Finland",
    "FR" => "France",
    "GF" => "French Guiana",
    "PF" => "French Polynesia",
    "TF" => "French Southern and Antarctic Lands",
    "GA" => "Gabon",
    "GM" => "Gambia",
    "GE" => "Georgia",
    "DE" => "Germany",
    "GH" => "Ghana",
    "GI" => "Gibraltar",
    "GR" => "Greece",
    "GL" => "Greenland",
    "GD" => "Grenada",
    "GP" => "Guadeloupe",
    "GU" => "Guam",
    "GT" => "Guatemala",
    "GN" => "Guinea",
    "GW" => "Guinea-Bissau",
    "GY" => "Guyana",
    "HT" => "Haiti",
    "HM" => "Heard Island and McDonald Islands",
    "HN" => "Honduras",
    "HK" => "Hong Kong SAR",
    "HU" => "Hungary",
    "IS" => "Iceland",
    "IN" => "India",
    "ID" => "Indonesia",
    "IR" => "Iran",
    "IQ" => "Iraq",
    "IE" => "Ireland",
    "IL" => "Israel",
    "IT" => "Italy",
    "JM" => "Jamaica",
    "JP" => "Japan",
    "JO" => "Jordan",
    "KZ" => "Kazakhstan",
    "KE" => "Kenya",
    "KI" => "Kiribati",
    "KR" => "Korea",
    "KW" => "Kuwait",
    "KG" => "Kyrgyzstan",
    "LA" => "Laos",
    "LV" => "Latvia",
    "LB" => "Lebanon",
    "LS" => "Lesotho",
    "LR" => "Liberia",
    "LY" => "Libya",
    "LI" => "Liechtenstein",
    "LT" => "Lithuania",
    "LU" => "Luxembourg",
    "MO" => "Macao SAR",
    "MK" => "Macedonia, Former Yugoslav Republic of",
    "MG" => "Madagascar",
    "MW" => "Malawi",
    "MY" => "Malaysia",
    "MV" => "Maldives",
    "ML" => "Mali",
    "MT" => "Malta",
    "MH" => "Marshall Islands",
    "MQ" => "Martinique",
    "MR" => "Mauritania",
    "MU" => "Mauritius",
    "YT" => "Mayotte",
    "MX" => "Mexico",
    "FM" => "Micronesia",
    "MD" => "Moldova",
    "MC" => "Monaco",
    "MN" => "Mongolia",
    "MS" => "Montserrat",
    "MA" => "Morocco",
    "MZ" => "Mozambique",
    "MM" => "Myanmar",
    "NA" => "Namibia",
    "NR" => "Nauru",
    "NP" => "Nepal",
    "NL" => "Netherlands",
    "AN" => "Netherlands Antilles",
    "NC" => "New Caledonia",
    "NZ" => "New Zealand",
    "NI" => "Nicaragua",
    "NE" => "Niger",
    "NG" => "Nigeria",
    "NU" => "Niue",
    "NF" => "Norfolk Island",
    "KP" => "North Korea",
    "MP" => "Northern Mariana Islands",
    "NO" => "Norway",
    "OM" => "Oman",
    "PK" => "Pakistan",
    "PW" => "Palau",
    "PA" => "Panama",
    "PG" => "Papua New Guinea",
    "PY" => "Paraguay",
    "PE" => "Peru",
    "PH" => "Philippines",
    "PN" => "Pitcairn Islands",
    "PL" => "Poland",
    "PT" => "Portugal",
    "PR" => "Puerto Rico",
    "QA" => "Qatar",
    "RE" => "Reunion",
    "RO" => "Romania",
    "RU" => "Russia",
    "RW" => "Rwanda",
    "WS" => "Samoa",
    "SM" => "San Marino",
    "ST" => "Sao Tome and Principe",
    "SA" => "Saudi Arabia",
    "SN" => "Senegal",
    "YU" => "Serbia and Montenegro",
    "SC" => "Seychelles",
    "SL" => "Sierra Leone",
    "SG" => "Singapore",
    "SK" => "Slovakia",
    "SI" => "Slovenia",
    "SB" => "Solomon Islands",
    "SO" => "Somalia",
    "ZA" => "South Africa",
    "GS" => "South Georgia and the South Sandwich Islands",
    "ES" => "Spain",
    "LK" => "Sri Lanka",
    "SH" => "St. Helena",
    "KN" => "St. Kitts and Nevis",
    "LC" => "St. Lucia",
    "PM" => "St. Pierre and Miquelon",
    "VC" => "St. Vincent and the Grenadines",
    "SD" => "Sudan",
    "SR" => "Suriname",
    "SJ" => "Svalbard and Jan Mayen",
    "SZ" => "Swaziland",
    "SE" => "Sweden",
    "CH" => "Switzerland",
    "SY" => "Syria",
    "TW" => "Taiwan",
    "TJ" => "Tajikistan",
    "TZ" => "Tanzania",
    "TH" => "Thailand",
    "TG" => "Togo",
    "TK" => "Tokelau",
    "TO" => "Tonga",
    "TT" => "Trinidad and Tobago",
    "TN" => "Tunisia",
    "TR" => "Turkey",
    "TM" => "Turkmenistan",
    "TC" => "Turks and Caicos Islands",
    "TV" => "Tuvalu",
    "UG" => "Uganda",
    "UA" => "Ukraine",
    "AE" => "United Arab Emirates",
    "UK" => "United Kingdom",
    "US" => "United States",
    "UM" => "United States Minor Outlying Islands",
    "UY" => "Uruguay",
    "UZ" => "Uzbekistan",
    "VU" => "Vanuatu",
    "VA" => "Vatican City",
    "VE" => "Venezuela",
    "VN" => "Viet Nam",
    "VG" => "Virgin Islands (British)",
    "VI" => "Virgin Islands",
    "WF" => "Wallis and Futuna",
    "YE" => "Yemen",
    "ZM" => "Zambia",
    "ZW" => "Zimbabwe"
);

?>
