<?php 
/*
*
*
*     Author: Alex Scott
*      Email: alex@cgi-central.net
*        Web: http://www.cgi-central.net
*    Details: CC CORE payment plugin
*    FileName $RCSfile: cc_core.inc.php,v $
*    Release: 2.1.8PRO ($Revision: 1.1.2.10 $)
*
* Please direct bug reports,suggestions or feedback to the cgi-central forums.
* http://www.cgi-central.net/forum/
*                                                                          
* aMember PRO is a commercial software. Any distribution is strictly prohibited.
*
*/

/// charge type constants (to pass to plugin function)
define('CC_CHARGE_TYPE_REGULAR', 1);
define('CC_CHARGE_TYPE_START_RECURRING', 2);
define('CC_CHARGE_TYPE_RECURRING', 3);
define('CC_CHARGE_TYPE_TEST', 4);

// result of processing (from the plugin call)
define('CC_RESULT_INTERNAL_ERROR', -1); // error of the plugin or paysystem
define('CC_RESULT_SUCCESS', 0);          // successfull
define('CC_RESULT_DECLINE_TEMP', 1); // temporary problem with customer cc info, 
                                   // like insufficient funds
define('CC_RESULT_DECLINE_PERM', 2); // permanent problem with customer cc info, 
                                   // like credit card is expired or locked


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 function cc_core_init($plugin)

 Adds necessary fields to member, product and possible payment
    
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
function cc_core_init($plugin){
    global $cc_core, $config;
    // get features array
    $pl = instantiate_plugin('payment', $plugin);
    $cc_core[$plugin] =  $pl;
    $features = $pl->get_plugin_features();
    ///
    add_paysystem_to_list(
    array(
                'paysys_id'   => $plugin,
                'title'       => $pl->config['title'] ? $pl->config['title'] : $features['title'],
                'description' => $pl->config['description'] ? $pl->config['description'] : $features['description'],
                'public'      => 1,
                'recurring'   => 1
            )
    );
    // add product fields 
    add_product_field(
                'is_recurring', 'Recurring Billing',
                'select', 'should user be charged automatically<br>
                 when subscription expires',
                '',
                array('options' => array(
                    '' => 'No',
                    1  => 'Yes'
                ))
    );
    add_product_field('trial1_days', 
        'Trial 1 Duration',
        'period',
        'Trial 1 duration'
        );

    add_product_field('trial1_price', 
        'Trial 1 Price',
        'money',
        'set 0 for free trial'
        );
    add_product_field('trial_group', 
        'Trial Group',
        'text',
        "If this field is filled-in, user will be unable to order the product<br>
         twice. It is extermelly useful for any trial product. This field<br>
         can have different values for different products, then 'trial history'<br>
         will be separate for these groups of products.<br>
         If your site offers only one level of membership,<br>
         just enter \"1\" to this field. 
        "
        );
    add_product_field('rebill_times', 
        'Recurring Times',
        'text',
        'Recurring Times. This is the number of payments which<br>
         will occur at the regular rate. If omitted, payment will<br>
         continue to recur at the regular rate until the subscription<br>
         is cancelled.<br>
         NOTE: not for all payment processing this option is working'
        );

    if ($features['currency'])
        add_product_field($plugin.'_currency',  $pl->config['title'] . " Currency",
             'select', "valid only for $plugin payments", '', $features['currency']);

    /// add member fields
    add_member_field(
            'cc_street',
            'Billing Street Address',
            'text', 
            "",
            '',
            array('hidden_anywhere' => 1)
    );
    add_member_field(
            'cc_city',
            'Billing City',
            'text', 
            "",
            ''
    );
    add_member_field(
            'cc_state',
            'Billing State',
            'select', 
            "",
            '', 
            array('options' => $config['state_options'])
    );
    add_member_field(
            'cc_zip',
            'Billing ZIP',
            'text', 
            "",
            ''
    );
    add_member_field(
            'cc_country',
            'Billing Country',
            'select', 
            "",
            '', 
            array('options' => $config['country_options'])
    );
    if ($features['name_f']){
        add_member_field('cc_name_f', 'Billing First Name', 'text', '');
        add_member_field('cc_name_l', 'Billing Last Name', 'text', '');
    } elseif ($features['name']) {
        add_member_field('cc_name', 'Billing Name', 'text', '');
    }
    if ($features['company'])
        add_member_field('cc_company', 'Billing Company', 'text', '');
    if ($features['phone']) 
        add_member_field('cc_phone', 'Billing Phone#', 'text', '');

    add_member_field(
            'cc',
            'Credit Card # (visible)',
            'readonly', 
            "credit card number (read-only)",
            '',
            array('hidden_anywhere' => 1)
    );

    add_member_field(
            'cc-hidden',
            'Credit Card # (crypted)',
            'hidden', 
            '',
            '',
            array('hidden_anywhere' => 1)
    );

    add_member_field(
            'cc-expire',
            'Credit Card Expire',
            'readonly', 
            'Expiration date (mmyy)',
            '',
            array('hidden_anywhere' => 1)
    );

    if ($features['code'] == 2)
        add_member_field(
                'cc-code-hidden',
                'Credit Card Code (crypted)',
                'hidden', 
                '',
                '',
                array('hidden_anywhere' => 1)
        );

    if ($features['type_options'])
        add_member_field('cc_type',  "CC Type",
             'select', "credit card type", '', array('options'=>$features['type_options']));

    /// setup hooks
    setup_plugin_hook('get_member_links', $plugin . '_get_member_links');
    setup_plugin_hook('daily', $plugin . '_rebill');
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 Add necessary config items to make aMember CP -> Setup -> PLUGIN
 working
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
function cc_core_add_config_items($plugin, $notebook_page){
    global $cc_core, $config;

    $pl = $cc_core[$plugin];
    $features = $pl->get_plugin_features();

    add_config_field("payment.$plugin.title", "Payment system title",
        'text', "to be displayed on signup.php and member.php pages",
        $notebook_page, 
        '','','',
        array('default' => $features['title']));
    add_config_field("payment.$plugin.description", "Payment system description",
        'text', "to be displayed on signup page",
        $notebook_page, 
        '','','',
        array('default' => $features['description']));
    
    add_config_field("payment.$plugin.mail_cancel_member", 'Send "Cancel" email to member',
        'select', "when member cancels recurring subscription, aMember can send<br>
        him an e-mail. You can enable or disable this feature.<br>
        E-Mail template is: amember/templates/cc/mail_cancel_member.txt
        ",
        $notebook_page, 
        '','','',
        array('options' => array(
            '' => 'No',
            1  => 'Yes'
        )));
    add_config_field("payment.$plugin.mail_cancel_admin", 'Send "Cancel" email to admin',
        'select', "when member cancels recurring subscription, aMember can send<br>
        an e-mail to admin. You can enable or disable this feature.<br>
        E-Mail template is: amember/templates/cc/mail_cancel_admin.txt
        ",
        $notebook_page, 
        '','','',
        array('options' => array(
            '' => 'No',
            1  => 'Yes'
        )));
/*
    add_config_field("payment.$plugin.internal_testing", 'Internal Test Mode',
        'select', "it allows you to run internal testing,<br>
        no transactions will be sent to payment processor<br>
        all transaction will result as you specify in this<br>
        option. PLEASE don't forget to disable this option<br>
        when you go live, else you will lose money very quick.<br>
        ",
        $notebook_page, 
        '','','',
        array('options' => array(
            0 => 'Internal testing disabled', 
            1 => 'Internal testing enabled - APPROVE all transactions',
            2 => 'Internal testing enabled - DECLINE all transactions'
        )));
*/
}

function get_cc_info_hash($member, $action){
    return md5($member['pass'].$action.($member_id * 12));
}

function clean_cc_info($member){
    global $db;
    $cc_fields = array(
        '', '-expire', '-hidden',
        '_code', '_type',
        '_name_f', '_name_l', '_name', 
        '_phone', '_company',
        '_street', '_city', '_state', '_zip','_country');
    $m = $db->get_user($member['member_id']);
    foreach ($cc_fields as $f)
        $m['data']['cc'.$f] = '';
    $db->update_user($m['member_id'], $m);
}

function cc_core_do_payment($plugin, $payment_id, $member_id, $product_id,
            $price, $begin_date, $expire_date, $vars){
    global $db, $cc_core, $config;
    $member = $db->get_user($member_id);
    $v = get_cc_info_hash($member, $action = "mfp");
    html_redirect("$config[root_surl]/plugins/payment/cc_core/cc.php?action=$action&payment_id=$payment_id&member_id=$member_id&v=$v",
        0, 'Redirecting you to the payment system', 
        'You will be automatically redirected to secure payment page in about 2 seconds');
}

function cc_core_get_payer_id($cc_info, $member){
    return "cc:" . md5($cc_info['cc_number']);
}

function cc_core_get_cancel_link($plugin, $payment_id){
    global $cc_core, $config, $db;
    $p = $db->get_payment($payment_id);
    $member = $db->get_user($p['member_id']);
    $action='cancel_recurring';
    $v = get_cc_info_hash($member, $action);
    if (!$p['data']['CANCELLED'])
        return "{$config[root_surl]}/plugins/payment/cc_core/cc.php?"
            ."action=$action&payment_id=$payment_id&"
            ."member_id={$p[member_id]}&v=$v";
}

function cc_core_get_member_links($plugin, $user){
    global $config;
    $action = "renew_cc";
    $v = get_cc_info_hash($user, $action);
    if ($user['data']['cc'])
        return array("$config[root_surl]/plugins/payment/cc_core/cc.php?action=$action&paysys_id=$plugin&member_id={$user[member_id]}&v=$v" => 'Update CC info');
}

function cc_core_check_rebill_times($times, $payment){
    global $db;
    if ($payment_id = $payment['data'][0]['RENEWAL_ORIG']) {
        $count = 1;
        do {
            $count++;
            $x = preg_split('/ /', $payment_id);
            $payment_id = $x[1];
            $payment = $db->get_payment($payment_id);
        } while ($payment_id = $payment['data'][0]['RENEWAL_ORIG']);
    } else {
        $payment_id = $payment['payment_id'];
        $count = 1; // made payments
    }
    return $count < $times;
}


function cc_core_rebill($plugin){
    global $config, $db, $t, $cc_core;
    if (!$config['use_cron'])
        $db->log_error("$plugin rebill can be run only with external cron");
    $dat = date('Y-m-d');
    $tomorrow = date('Y-m-d', time() + 3600 * 24);

    $pl = $cc_core[$plugin];
    if (!is_object($pl)) {
        $db->log_error("$plugin is not handled by cc_core plugin");
        return;
    }
    
    $payments = $db->get_expired_payments($dat, $dat, $plugin);
    $renewed = array();
    $log = "$plugin Rebill\n";
    foreach ($payments as $p){
        if ($p['data']['CANCELLED']) 
            continue;
        $member_id = $p['member_id'];
        $member = $db->get_user($member_id);
        $product_id = $p['product_id'];
        if ($renewed[$member_id][$product_id]++) continue;
        $product = & get_product($product_id);
        if (!$product->config['is_recurring']) continue;
        if ($product->config['rebill_times'] &&
            !cc_core_check_rebill_times($product->config['rebill_times'], $p))
            continue;
        $vars = array(
            'RENEWAL_ORIG' => "RENEWAL_ORIG: $p[payment_id]"
        );
        $payment_id = $db->add_waiting_payment($member_id, $product_id, 
             $plugin, $product->config['price'], 
             $dat, $product->get_expire($dat), 
             $vars);

        $payment = $db->get_payment($payment_id);
        $cc_info = $member['data'];
        $cc_info['cc_number'] = amember_decrypt($cc_info['cc-hidden']);
        $x = list($res, $err_msg, $receipt_id, $log) = $pl->cc_bill($cc_info, 
            $member, $payment['amount'],
            $product[$payment['paysys_id'] . '_currency'], 
            $product['title'], CC_CHARGE_TYPE_RECURRING, 
            $payment['payment_id'], $payment);
        foreach ($log as $v)
            $payment['data'][] = $v;
        $db->update_payment($payment['payment_id'], $payment);
        switch ($res){
        case CC_RESULT_SUCCESS:
            $err = $db->finish_waiting_payment(
                $payment['payment_id'], $payment['paysys_id'], 
                    $receipt_id, $payment['amount'], '', cc_core_get_payer_id($vars, $member));
            if ($err) {
                $db->log_error($err . ": payment_id = $payment[payment_id] (rebilling)");
            }
        break;
        case CC_RESULT_INTERNAL_ERROR:
        case CC_RESULT_DECLIME_TEMP:
            mail_rebill_failed_member($member, $payment_id, $product, "$err_msg");
        break;
        case CC_RESULT_DECLINE_PERM:
            mail_rebill_failed_member($member, $payment_id, $product, "$err_msg");
            clean_cc_info($member);
        break;
        default:
            $db->log_error("Unknown return from plugin_bill: $res");
        };
    }
}

function cc_core_get_url($url, $post=''){
    global $db;
    if (extension_loaded("curl")){
        $ch=curl_init($url);
        curl_setopt($ch, CURLOPT_HEADER, 0);
        if ($post)  {
            curl_setopt($ch, CURLOPT_POSTFIELDS, $post);  
        }
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);    
        $buffer = curl_exec($ch);                       
        curl_close($ch);
        return $buffer;
    } else {
        global $config;
        $curl = $config['curl'];
        if (!strlen($curl)) {
            $db->log_error("cURL path is not set - cc transaction cannot be completed");
            return;
        }
        if (substr(php_uname(), 0, 7) == "Windows") {
            if ($post)
            $ret = `$curl -d "$post" "$url"`;
            else
            $ret = `$curl "$url"`;
        } else {
            $url  = escapeshellarg($url);
            $post = escapeshellarg($post);
            if ($post)
            $ret = `$curl -d $post $url`;
            else
            $ret = `$curl $url`;
        }
        return $ret;
    }
}


?>